import * as react from 'react';

/**
 * Custom hook that returns a function indicating whether the component
 * is mounted or not.
 *
 * @returns A function that returns a boolean value indicating whether
 *          the component is mounted.
 *
 * @example
 * const isMounted = useIsMounted();
 * console.log(isMounted()); // true
 *
 */
declare function useIsMounted(): () => boolean;

interface StorageProperties<T> {
    /**
     * Storage key.
     */
    key: string;
    /**
     * Default value that will be set if value is not found in storage.
     */
    initialValue?: T;
}
/**
 *
 * @example
 * import { useLocalStorage } from '@versini/ui-hooks';
 * const [value, setValue, resetValue, removeValue] = useLocalStorage({
 *   key: 'gpt-model',
 *   initialValue: 'gpt-3',
 * });
 *
 * setValue('gpt-4'); ==> "gpt-4"
 * setValue((current) => (current === 'gpt-3' ? 'gpt-4' : 'gpt-3'));
 * resetValue(); ==> "gpt-3"
 * removeValue(); ==> null
 */
declare function useLocalStorage<T>({ key, initialValue, }: StorageProperties<T>): any[];

/**
 * React utility to merge refs.
 *
 * When developing low level UI components, it is common to have to use a local
 * ref but also support an external one using React.forwardRef. Natively, React
 * does not offer a way to set two refs inside the ref property.
 *
 * @param Array of refs (object, function, etc.)
 *
 * @example
 *
 * const Example = React.forwardRef(function Example(props, ref) {
 *   const localRef = React.useRef();
 *   const mergedRefs = useMergeRefs([localRef, ref]);
 *
 *   return <div ref={mergedRefs} />;
 * });
 *
 */
declare function useMergeRefs<T = any>(refs: Array<React.MutableRefObject<T> | React.LegacyRef<T> | undefined | null>): React.RefCallback<T>;

type ObserverRect = Omit<DOMRectReadOnly, "toJSON">;
/**
 * A custom hook that uses the ResizeObserver API to track the size changes of a DOM element.
 *
 * @template T - The type of the DOM element being observed.
 * @param {ResizeObserverOptions} [options] - The options to configure the ResizeObserver.
 * @returns {[React.RefObject<T>, ObserverRect]} - A tuple containing the ref object and
 *                                                the observed rectangle.
 * @example
 *
 * const [rightElementRef, rect] = useResizeObserver<HTMLDivElement>();
 * <div ref={componentRef}>
 *   Observed: <code>{JSON.stringify(rect)}</code>
 * </div>
 */
declare function useResizeObserver<T extends HTMLElement = any>(options?: ResizeObserverOptions): readonly [react.RefObject<T>, ObserverRect];

/**
 * MIT License
 *
 * Copyright (c) 2021 Vitaly Rtishchev
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
interface UseUncontrolledInput<T> {
    /** Value for controlled state */
    value?: T;
    /** Initial value for uncontrolled state */
    defaultValue?: T;
    /** Final value for uncontrolled state when value and defaultValue are not provided */
    finalValue?: T;
    /** Controlled state onChange handler */
    onChange?: (value: T) => void;
}
declare function useUncontrolled<T>({ value, defaultValue, finalValue, onChange, }: UseUncontrolledInput<T>): [T, (value: T) => void, boolean];

/**
 * Hook that generates a unique id that will retain its value
 * during the lifecycle of the calling functional component.
 *
 * The parameters are either
 * - nothing: a unique id will simply be generated with no prefix.
 * - a string or a number: a prefix to prepend to the generated id.
 * - an object with the following props:
 *    - id: if this is a number or a string, it will be returned as is
 *    - prefix: prefix to prepend to the generated id.
 *
 * @param {string | number | object} options
 * @returns a unique id
 *
 * @examples
 *
 * const someId = useUniqueId();
 * // -> someId = "1j3h4f5"
 *
 * const errorId = useUniqueId("av-text-input-error-");
 * // -> errorId = "av-text-input-error-1j3h4f5"
 *
 * const inputId = useUniqueId({ id: 42, prefix: "av-text-input-" });
 * // -> inputId = "av-text-input-42"
 *
 * const inputHintId = useUniqueId({
 *   prefix: "av-text-input-hint-",
 * });
 * // -> inputHintId = "av-text-input-hint-1j3h4f5"
 *
 */
type UseUniqueIdOptions = string | number | {
    id?: string | number;
    prefix?: string;
};
declare function useUniqueId(options?: UseUniqueIdOptions): string | undefined;

export { useIsMounted, useLocalStorage, useMergeRefs, useResizeObserver, useUncontrolled, useUniqueId };
